<?php

declare(strict_types=1);

namespace Drupal\aws\Entity;

use Aws\Credentials\Credentials;
use Aws\Credentials\CredentialProvider;
use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\encrypt\EncryptServiceInterface;

/**
 * Defines the AWS Profile entity.
 *
 * @ConfigEntityType(
 *   id = "aws_profile",
 *   label = @Translation("AWS Profile"),
 *   label_collection = @Translation("AWS Profiles"),
 *   label_singular = @Translation("AWS profile"),
 *   label_plural = @Translation("AWS profiles"),
 *   label_count = @PluralTranslation(
 *     singular = "@count AWS profile",
 *     plural = "@count AWS profiles",
 *   ),
 *   handlers = {
 *     "storage" = "Drupal\aws\Entity\Storage\ProfileStorage",
 *     "list_builder" = "Drupal\aws\Entity\ListBuilder\ProfileListBuilder",
 *     "form" = {
 *       "default" = "Drupal\aws\Entity\Form\ProfileForm",
 *       "edit" = "Drupal\aws\Entity\Form\ProfileForm",
 *       "delete" = "Drupal\aws\Entity\Form\ProfileDeleteConfirmForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *   },
 *   admin_permission = "administer aws",
 *   config_prefix = "profile",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *   },
 *   config_export = {
 *     "id",
 *     "name",
 *     "provider",
 *     "provider_config",
 *     "region",
 *     "encryption_profile",
 *     "default",
 *   },
 *   links = {
 *     "canonical" = "/admin/config/services/aws/profile/{aws_profile}",
 *     "add-form" = "/admin/config/services/aws/add-profile",
 *     "edit-form" = "/admin/config/services/aws/profile/{aws_profile}/edit",
 *     "delete-form" = "/admin/config/services/aws/profile/{aws_profile}/delete",
 *     "collection" = "/admin/config/services/aws/profiles",
 *   }
 * )
 */
class Profile extends ConfigEntityBase implements ProfileInterface {

  /**
   * The ID of the profile.
   *
   * @var string
   */
  protected $id;

  /**
   * The name of the profile.
   *
   * @var string
   */
  protected $name;

  /**
   * The credential provider of the profile.
   *
   * @var string
   */
  protected $provider;

  /**
   * Configuration values for the credential provider.
   *
   * @var array
   */
  protected $provider_config;

  /**
   * Whether the profile is the default or not.
   *
   * @var int
   */
  protected $default;

  /**
   * The region of the profile.
   *
   * @var string
   */
  protected $region;

  /**
   * The encryption profile for the profile.
   *
   * @var string
   */
  protected $encryption_profile;

  /**
   * The encryption service.
   *
   * @var \Drupal\encrypt\EncryptServiceInterface|null
   */
  protected $encryption;

  /**
   * Constructs an Entity object.
   *
   * @param array $values
   *   An array of values to set, keyed by property name.
   * @param string $entity_type
   *   The type of the entity to create.
   * @param \Drupal\encrypt\EncryptServiceInterface|null $encryption
   *   The encryption service.
   */
  public function __construct(array $values, $entity_type, ?EncryptServiceInterface $encryption) {
    parent::__construct($values, $entity_type);
    $this->encryption = $encryption;
  }

  /**
   * {@inheritdoc}
   */
  public function getProvider() {
    return $this->provider;
  }

  /**
   * {@inheritdoc}
   */
  public function setProvider(string $provider) {
    $this->provider = $provider;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getProviderConfig() {
    $secret = $this->provider_config['secret'] ?? NULL;

    if ($secret && $this->encryption && $this->encryption_profile != '_none') {
      $storage = $this->entityTypeManager()->getStorage('encryption_profile');
      /** @var \Drupal\encrypt\EncryptionProfileInterface  $encryption_profile */
      $encryption_profile = $storage->load($this->encryption_profile);
      $decrypted_secret = $this->encryption->decrypt($secret, $encryption_profile);
      $this->provider_config['secret'] = $decrypted_secret;
    }

    return $this->provider_config;
  }

  /**
   * {@inheritdoc}
   */
  public function setProviderConfig(array $config) {
    $secret = $config['secret'] ?? NULL;

    if ($secret && $this->encryption && $this->encryption_profile != '_none') {
      $storage = $this->entityTypeManager()->getStorage('encryption_profile');
      /** @var \Drupal\encrypt\EncryptionProfileInterface  $encryption_profile */
      $encryption_profile = $storage->load($this->encryption_profile);
      $encrypted_secret = $this->encryption->encrypt($secret, $encryption_profile);
      $config['secret'] = $encrypted_secret;
    }

    $this->provider_config = array_filter($config);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRegion() {
    return $this->region;
  }

  /**
   * {@inheritdoc}
   */
  public function setRegion(string $region) {
    $this->region = $region;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getEncryptionProfile() {
    return $this->encryption_profile;
  }

  /**
   * {@inheritdoc}
   */
  public function setEncryptionProfile(string $encryption_profile) {
    $this->encryption_profile = $encryption_profile;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isDefault() {
    return (bool) $this->default;
  }

  /**
   * {@inheritdoc}
   */
  public function setDefault(bool $default) {
    $this->default = (int) $default;
    return $this;
  }

  /**
   * Get the Credential Provider for the profile.
   *
   * @return \Aws\Credentials\CredentialProvider
   *   The Credential Provider for the profile.
   */
  protected function getCredentialProvider() {
    switch ($this->provider) {
      case ProfileInterface::PROVIDER_ENV:
        $provider = CredentialProvider::env();
        break;

      case ProfileInterface::PROVIDER_FILE:
        $config = $this->getProviderConfig();
        $profile = $config['profile'] ?? NULL;
        $provider = CredentialProvider::ini($profile);
        break;

      case ProfileInterface::PROVIDER_KEYS:
        $config = $this->getProviderConfig();
        $credentials = new Credentials($config['key_id'], $config['secret']);
        $provider = CredentialProvider::fromCredentials($credentials);
        break;
    }

    return $provider;
  }

  /**
   * {@inheritdoc}
   */
  public function getClientArgs(string $version = 'latest') {
    return [
      'credentials' => $this->getCredentialProvider(),
      'region' => $this->getRegion(),
      'version' => $version,
    ];
  }

}
